<?php
declare(strict_types=1);

namespace Catequil\Cache\Adapter;

use Laminas\ServiceManager\Factory\FactoryInterface;
use Predis\ClientInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;
use Symfony\Component\Cache\Adapter\RedisAdapter;
use function array_merge;

/**
 * Class RedisAdapterFactory
 *
 * @package Catequil\Cache\Adapter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-cache
 */
final class RedisAdapterFactory implements FactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return RedisAdapter
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): RedisAdapter
    {
        /** @var ClientInterface $redis */
        $redis = $container->get(ClientInterface::class);
        $redis->select($options['database'] ?? 0);

        return new RedisAdapter(
            $redis,
            preg_replace('/\W/', '_', $options['namespace'] ?? ''),
            $options['default_lifetime'] ?? 0,
        );
    }
}