<?php
declare(strict_types=1);

namespace Catequil\Cache;

use Catequil\Cache\Service\CacheManager;
use Catequil\Tool\Metadata\MetadataManager;
use Predis\Client;
use Predis\ClientInterface;
use Symfony\Component\Cache\Adapter\RedisAdapter;

/**
 * Class ConfigProvider
 *
 * @package Catequil\Cache
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-cache
 */
final class ConfigProvider
{
    public function __invoke(): array
    {
        return [
            CacheManager::class => $this->getCaches(),
            'dependencies' => $this->getDependencies(),
            'redis' => $this->getRedisConfig(),
            MetadataManager::class => [
                'paths' => [
                    __DIR__,
                ],
            ],
        ];
    }

    public function getCaches(): array
    {
        return [
            'cache.default' => [
                'adapter' => RedisAdapter::class,
                'options' => [
                    'namespace' => __DIR__,
                    'database' => 0,
                ],
            ],
        ];
    }

    public function getRedisConfig(): array
    {
        return [
            'connection' => [
                'scheme' => 'tcp',
                'host' => '127.0.0.1',
                'port' => 6379,
            ],
            'options' => [
                'prefix' => '',
                'exceptions' => true,
            ],
        ];
    }

    public function getDependencies(): array
    {
        return [
            'aliases' => [
                ClientInterface::class => Client::class,
                'cache' => 'cache.default',
            ],
            'factories' => [
                'cache.default' => Service\CacheAbstractFactory::class,
                Client::class => Service\RedisFactory::class,
                Service\CachePluginManager::class => Service\CachePluginManagerFactory::class,
            ],
        ];
    }
}