<?php
declare(strict_types=1);

namespace Catequil\Cache\Console;

use Catequil\Cache\Service\CacheManager;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Throwable;

/**
 * Class ClearCache
 *
 * @package Catequil\Cache\Console
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-cache
 */
#[AsCommand('cache:clear')]
final class ClearCache extends Command
{
    private CacheManager $caches;

    /**
     * @param CacheManager $caches
     */
    public function __construct(CacheManager $caches)
    {
        parent::__construct();
        $this->caches = $caches;
    }

    protected function configure()
    {
        $this->addArgument('cache', InputArgument::OPTIONAL, 'The cache to clear');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $cache = $input->getArgument('cache');

        try {
            if ($cache) {
                $output->writeln('Clearing cache ' . $cache . ' ...');
                $this->caches->clear($cache);
            } else {
                $output->writeln('Clearing all caches ...');
                $this->caches->clearAll();
            }
        } catch (Throwable $e) {
            $output->writeln('ERROR: ' . $e->getMessage());
            return self::FAILURE;
        }

        $output->writeln('done');
        return self::SUCCESS;
    }
}
