<?php
declare(strict_types=1);

namespace Catequil\Cache\Service;

use Laminas\ServiceManager\Factory\AbstractFactoryInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;
use Symfony\Contracts\Cache\CacheInterface;

/**
 * Class CacheAbstractFactory
 *
 * @package Catequil\Cache\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-cache
 */
final class CacheAbstractFactory implements AbstractFactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     *
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName): bool
    {
        return $this->getCacheConfig($container, $requestedName) !== null;
    }

    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return CacheInterface
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): CacheInterface
    {
        $config = $this->getCacheConfig($container, $requestedName);
        /** @var CachePluginManager $adapters */
        $adapters = $container->get(CachePluginManager::class);

        $options = $config['options'] ?? [];

        if (isset($options['namespace'])) {
            $options['namespace'] = \preg_replace(
                '/\W+/', '_', $options['namespace']
            );
        }

        return $adapters->get($config['adapter'], $options);
    }

    /**
     * @param ContainerInterface $container
     * @param string $name
     *
     * @return array|null
     *
     * @noinspection PhpDocMissingThrowsInspection
     */
    private function getCacheConfig(ContainerInterface $container, string $name): ?array
    {
        /** @noinspection PhpUnhandledExceptionInspection */
        $config = $container->has('config')
            ? ($container->get('config')[CacheManager::class] ?? [])
            : [];
        return $config[$name] ?? null;
    }
}