<?php
declare(strict_types=1);

namespace Catequil\Cache\Service;

use Psr\Cache\CacheItemPoolInterface;
use Psr\Container\ContainerInterface;
use RuntimeException;
use Symfony\Component\Cache\Adapter\AbstractAdapter;
use Symfony\Component\Cache\Adapter\AdapterInterface;
use Symfony\Contracts\Cache\CacheInterface;
use function array_keys;
use function sprintf;

/**
 * Class CacheManager
 *
 * @package Catequil\Cache\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-cache
 */
final class CacheManager
{
    private ContainerInterface $container;

    /**
     * @param ContainerInterface $container
     */
    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    public function has(string $name): bool
    {
        return $this->container->has($name);
    }

    public function get(string $name = 'cache'): AdapterInterface|CacheInterface|CacheItemPoolInterface
    {
        if (!$this->container->has($name)) {
            throw new RuntimeException(
                sprintf(
                    'Invalid cache name %s',
                    $name
                )
            );
        }

        /** @noinspection PhpUnhandledExceptionInspection */
        return $this->container->get($name);
    }

    public function clearAll(): void
    {
        /** @noinspection PhpUnhandledExceptionInspection */
        $config = $this->container->has('config')
            ? $this->container->get('config')[CacheManager::class]
            : [];

        foreach (array_keys($config) as $name) {
            $this->clear($name);
        }
    }

    public function clear(string $name = 'cache'): void
    {
        $cache = $this->get($name);
        $cache->clear();
    }
}