<?php
declare(strict_types=1);

namespace Catequil\Cache\Service;

use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Exception\ServiceNotFoundException;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Predis\Client;
use Predis\ClientInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;

/**
 * Class RedisConnectionFactory
 *
 * @package Catequil\Cache\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-cache
 */
final class RedisFactory implements FactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return ClientInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): ClientInterface
    {
        /** @noinspection PhpUnhandledExceptionInspection */
        $config = $container->has('config') ? ($container->get('config')['redis'] ?? []): [];

        return new Client(
            $config['connection'] ?? [],
            $config['options'] ?? []
        );
    }
}